@@ -31,7 +31,7 @@ master_doc = 'index'
 
 # General information about the project.
 project = u'Motor'
-copyright = u'2016 MongoDB, Inc.'
+copyright = u'2016-present MongoDB, Inc.'
 
 # The version info for the project you're documenting, acts as replacement for
 # |version| and |release|, also used in various other places throughout the
@@ -85,7 +85,24 @@ from tornado.ioloop import IOLoop
 
 import pymongo
 from pymongo.mongo_client import MongoClient
+
 sync_client = MongoClient()
+ismaster = sync_client.admin.command('isMaster')
+server_info = sync_client.server_info()
+
+if 'setName' in ismaster:
+    raise Exception(
+        "Run doctests with standalone MongoDB 3.6 server, not a replica set")
+
+if ismaster.get('msg') == 'isdbgrid':
+    raise Exception(
+        "Run doctests with standalone MongoDB 3.6 server, not mongos")
+
+if server_info['versionArray'][:2] != [3, 6]:
+    raise Exception(
+        "Run doctests with standalone MongoDB 3.6 server, not %s" % (
+            server_info['version'], ))
+
 sync_client.drop_database("doctest_test")
 db = sync_client.doctest_test
 
@@ -95,6 +112,8 @@ from motor import MotorClient
 
 # -- Options for HTML output ---------------------------------------------------
 
+html_copy_source = False
+
 # Theme gratefully vendored from CPython source.
 html_theme = "pydoctheme"
 html_theme_path = ["."]
@@ -183,7 +202,7 @@ latex_documents = [
 autodoc_default_flags = ['inherited-members']
 autodoc_member_order = 'groupwise'
 
-pymongo_inventory = ('http://api.mongodb.org/python/%s/' % pymongo_version,
+pymongo_inventory = ('http://api.mongodb.com/python/%s/' % pymongo_version,
                      None)
 
 intersphinx_mapping = {
