@@ -1,4 +1,4 @@
-# Copyright 2011-2015 MongoDB, Inc.
+# Copyright 2011-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,14 +20,13 @@ import pymongo
 
 from motor.motor_py3_compat import text_type
 
-version_tuple = (1, 1)
+version_tuple = (1, 2, 1)
 
 
 def get_version_string():
-    if isinstance(version_tuple[-1], text_type):
-        return '.'.join(map(str, version_tuple[:-1])) + version_tuple[-1]
     return '.'.join(map(str, version_tuple))
 
+
 version = get_version_string()
 """Current version of Motor."""
 
@@ -45,17 +44,6 @@ if pymongo.version_tuple[:2] < pymongo_required:
 
     raise ImportError(msg)
 
-# MotorClient runs getaddrinfo on a thread. If the hostname is unicode,
-# getaddrinfo attempts to import encodings.idna. If this is done at
-# module-import time, perhaps with "loop.run_sync(MotorClient.open)" at
-# module scope, the import lock is already held by the main thread, leading to
-# AutoReconnect. Avoid it by caching the idna encoder on the main thread now.
-#
-# Only required in Python 2 and Tornado 3.
-#
-# See also https://github.com/tornadoweb/tornado/pull/964
-u'foo'.encode('idna')
-
 try:
     import tornado
 except ImportError:
