@@ -1,4 +1,4 @@
-# Copyright 2011-2015 MongoDB, Inc.
+# Copyright 2011-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,6 +31,7 @@ import pymongo.son_manipulator
 
 from pymongo.bulk import BulkOperationBuilder
 from pymongo.database import Database
+from pymongo.change_stream import ChangeStream
 from pymongo.collection import Collection
 from pymongo.cursor import Cursor, _QUERY_OPTIONS
 from pymongo.command_cursor import CommandCursor
@@ -54,7 +55,6 @@ except ImportError:
     ssl = None
     HAS_SSL = False
 
-PY352 = sys.version_info >= (3, 5, 2)
 PY35 = sys.version_info >= (3, 5)
 
 
@@ -98,6 +98,8 @@ class AgnosticClient(AgnosticBaseProperties):
     is_mongos                = ReadOnlyProperty()
     is_primary               = ReadOnlyProperty()
     kill_cursors             = AsyncCommand()
+    list_databases           = AsyncRead().wrap(CommandCursor)
+    list_database_names      = AsyncRead()
     local_threshold_ms       = ReadOnlyProperty()
     max_bson_size            = ReadOnlyProperty()
     max_idle_time_ms         = ReadOnlyProperty()
@@ -109,9 +111,11 @@ class AgnosticClient(AgnosticBaseProperties):
     PORT                     = ReadOnlyProperty()
     primary                  = ReadOnlyProperty()
     read_concern             = ReadOnlyProperty()
+    retry_writes             = ReadOnlyProperty()
     secondaries              = ReadOnlyProperty()
     server_info              = AsyncRead()
     server_selection_timeout = ReadOnlyProperty()
+    start_session            = AsyncRead(doc=start_session_doc)
     unlock                   = AsyncCommand()
 
     def __init__(self, *args, **kwargs):
@@ -157,50 +161,61 @@ class AgnosticClient(AgnosticBaseProperties):
 
         return db_class(self, name)
 
-    def wrap(self, db):
-        # Replace pymongo.database.Database with MotorDatabase.
-        db_class = create_class_with_framework(
-            AgnosticDatabase,
-            self._framework,
-            self.__module__)
+    def wrap(self, obj):
+        if obj.__class__ == Database:
+            db_class = create_class_with_framework(
+                AgnosticDatabase,
+                self._framework,
+                self.__module__)
 
-        return db_class(self, db.name, _delegate=db)
+            return db_class(self, obj.name, _delegate=obj)
+        elif obj.__class__ == CommandCursor:
+            command_cursor_class = create_class_with_framework(
+                AgnosticCommandCursor,
+                self._framework,
+                self.__module__)
+
+            return command_cursor_class(obj, self)
 
 
 class AgnosticDatabase(AgnosticBaseProperties):
     __motor_class_name__ = 'MotorDatabase'
     __delegate_class__ = Database
 
-    add_user            = AsyncCommand()
-    authenticate        = AsyncCommand()
-    collection_names    = AsyncRead()
-    command             = AsyncCommand(doc=cmd_doc)
-    create_collection   = AsyncCommand().wrap(Collection)
-    current_op          = AsyncRead()
-    dereference         = AsyncRead()
-    drop_collection     = AsyncCommand().unwrap('MotorCollection')
-    error               = AsyncRead(doc="OBSOLETE")
-    eval                = AsyncCommand()
-    get_collection      = DelegateMethod().wrap(Collection)
-    last_status         = AsyncRead(doc="OBSOLETE")
-    logout              = AsyncCommand()
-    name                = ReadOnlyProperty()
-    previous_error      = AsyncRead(doc="OBSOLETE")
-    profiling_info      = AsyncRead()
-    profiling_level     = AsyncRead()
-    remove_user         = AsyncCommand()
-    reset_error_history = AsyncCommand(doc="OBSOLETE")
-    set_profiling_level = AsyncCommand()
-    validate_collection = AsyncRead().unwrap('MotorCollection')
+    add_user              = AsyncCommand()
+    authenticate          = AsyncCommand()
+    collection_names      = AsyncRead()
+    command               = AsyncCommand(doc=cmd_doc)
+    create_collection     = AsyncCommand().wrap(Collection)
+    current_op            = AsyncRead()
+    dereference           = AsyncRead()
+    drop_collection       = AsyncCommand().unwrap('MotorCollection')
+    error                 = AsyncRead(doc="OBSOLETE")
+    eval                  = AsyncCommand()
+    get_collection        = DelegateMethod().wrap(Collection)
+    last_status           = AsyncRead(doc="OBSOLETE")
+    list_collection_names = AsyncRead()
+    list_collections      = AsyncRead()
+    logout                = AsyncCommand()
+    name                  = ReadOnlyProperty()
+    previous_error        = AsyncRead(doc="OBSOLETE")
+    profiling_info        = AsyncRead()
+    profiling_level       = AsyncRead()
+    remove_user           = AsyncCommand()
+    reset_error_history   = AsyncCommand(doc="OBSOLETE")
+    set_profiling_level   = AsyncCommand()
+    validate_collection   = AsyncRead().unwrap('MotorCollection')
 
     incoming_manipulators         = ReadOnlyProperty()
     incoming_copying_manipulators = ReadOnlyProperty()
     outgoing_manipulators         = ReadOnlyProperty()
     outgoing_copying_manipulators = ReadOnlyProperty()
 
-    def __init__(self, client, name, _delegate=None):
+    def __init__(self, client, name, **kwargs):
         self._client = client
-        delegate = _delegate or Database(client.delegate, name)
+        delegate = kwargs.get('_delegate') or Database(
+            client.delegate, name, **kwargs)
+
         super(self.__class__, self).__init__(delegate)
 
     @property
@@ -277,47 +292,49 @@ class AgnosticCollection(AgnosticBaseProperties):
     __motor_class_name__ = 'MotorCollection'
     __delegate_class__ = Collection
 
-    bulk_write           = AsyncCommand()
+    bulk_write           = AsyncCommand(doc=bulk_write_doc)
     count                = AsyncRead()
     create_index         = AsyncCommand()
-    create_indexes       = AsyncCommand()
-    delete_many          = AsyncCommand()
-    delete_one           = AsyncCommand()
+    create_indexes       = AsyncCommand(doc=create_indexes_doc)
+    delete_many          = AsyncCommand(doc=delete_many_doc)
+    delete_one           = AsyncCommand(doc=delete_one_doc)
     distinct             = AsyncRead()
-    drop                 = AsyncCommand()
+    drop                 = AsyncCommand(doc=drop_doc)
     drop_index           = AsyncCommand()
     drop_indexes         = AsyncCommand()
     ensure_index         = AsyncCommand()
     find_and_modify      = AsyncCommand()
-    find_one             = AsyncRead()
-    find_one_and_delete  = AsyncCommand()
-    find_one_and_replace = AsyncCommand()
-    find_one_and_update  = AsyncCommand()
+    find_one             = AsyncRead(doc=find_one_doc)
+    find_one_and_delete  = AsyncCommand(doc=find_one_and_delete_doc)
+    find_one_and_replace = AsyncCommand(doc=find_one_and_replace_doc)
+    find_one_and_update  = AsyncCommand(doc=find_one_and_update_doc)
     full_name            = ReadOnlyProperty()
     group                = AsyncRead()
-    index_information    = AsyncRead()
+    index_information    = AsyncRead(doc=index_information_doc)
     inline_map_reduce    = AsyncRead()
     insert               = AsyncWrite()
-    insert_many          = AsyncWrite()
-    insert_one           = AsyncCommand()
-    list_indexes         = AsyncRead()
+    insert_many          = AsyncWrite(doc=insert_many_doc)
+    insert_one           = AsyncCommand(doc=insert_one_doc)
     map_reduce           = AsyncCommand(doc=mr_doc).wrap(Collection)
     name                 = ReadOnlyProperty()
     options              = AsyncRead()
     reindex              = AsyncCommand()
     remove               = AsyncWrite()
     rename               = AsyncCommand()
-    replace_one          = AsyncCommand()
+    replace_one          = AsyncCommand(doc=replace_one_doc)
     save                 = AsyncWrite()
     update               = AsyncWrite(doc=update_doc)
-    update_many          = AsyncCommand()
-    update_one           = AsyncCommand()
+    update_many          = AsyncCommand(doc=update_many_doc)
+    update_one           = AsyncCommand(doc=update_one_doc)
     with_options         = DelegateMethod().wrap(Collection)
 
-    _async_aggregate  = AsyncRead(attr_name='aggregate')
-    __parallel_scan   = AsyncRead(attr_name='parallel_scan')
+    _async_aggregate    = AsyncRead(attr_name='aggregate')
+    _async_list_indexes = AsyncRead(attr_name='list_indexes')
+    __parallel_scan     = AsyncRead(attr_name='parallel_scan')
 
-    def __init__(self, database, name, _delegate=None):
+    def __init__(self, database, name, codec_options=None,
+                 read_preference=None, write_concern=None, read_concern=None,
+                 _delegate=None):
         db_class = create_class_with_framework(
             AgnosticDatabase, self._framework, self.__module__)
 
@@ -325,7 +342,11 @@ class AgnosticCollection(AgnosticBaseProperties):
             raise TypeError("First argument to MotorCollection must be "
                             "MotorDatabase, not %r" % database)
 
-        delegate = _delegate or Collection(database.delegate, name)
+        delegate = _delegate or Collection(
+            database.delegate, name, codec_options=codec_options,
+            read_preference=read_preference, write_concern=write_concern,
+            read_concern=read_concern)
+
         super(self.__class__, self).__init__(delegate)
         self.database = database
 
@@ -381,6 +402,9 @@ class AgnosticCollection(AgnosticBaseProperties):
 
         :Parameters:
           - `pipeline`: a single command or list of aggregation commands
+          - `session` (optional): a
+            :class:`~pymongo.client_session.ClientSession`, created with
+            :meth:`~MotorClient.start_session`.
           - `**kwargs`: send arbitrary parameters to the aggregate command
 
         Returns a :class:`MotorCommandCursor` that can be iterated like a
@@ -425,10 +449,152 @@ class AgnosticCollection(AgnosticBaseProperties):
 
             kwargs.setdefault('cursor', {})
             cursor_class = create_class_with_framework(
-                AgnosticAggregationCursor, self._framework, self.__module__)
+                AgnosticLatentCommandCursor, self._framework, self.__module__)
 
             # Latent cursor that will send initial command on first "async for".
-            return cursor_class(self, pipeline, **kwargs)
+            return cursor_class(self, self._async_aggregate, pipeline, **kwargs)
+
+    def watch(self, pipeline=None, full_document='default', resume_after=None,
+              max_await_time_ms=None, batch_size=None, collation=None,
+              session=None):
+        """Watch changes on this collection.
+
+        Returns a :class:`~MotorChangeStream` cursor which iterates over changes
+        on this collection. Introduced in MongoDB 3.6.
+
+        A change stream continues waiting indefinitely for matching change
+        events. Code like the following allows a program to cancel the change
+        stream and exit.
+
+        .. code-block:: python3
+
+          change_stream = None
+
+          async def watch_collection():
+              global change_stream
+
+              # Using the change stream in an "async with" block
+              # ensures it is canceled promptly if your code breaks
+              # from the loop or throws an exception.
+              async with db.collection.watch() as change_stream:
+                  async for change in stream:
+                      print(change)
+
+          # Tornado
+          from tornado.ioloop import IOLoop
+
+          def main():
+              loop = IOLoop.current()
+              # Start watching collection for changes.
+              loop.add_callback(watch_collection)
+              try:
+                  loop.start()
+              except KeyboardInterrupt:
+                  pass
+              finally:
+                  if change_stream is not None:
+                      change_stream.close()
+
+          # asyncio
+          from asyncio import get_event_loop
+
+          def main():
+              loop = get_event_loop()
+              task = loop.create_task(watch_collection)
+
+              try:
+                  loop.run_forever()
+              except KeyboardInterrupt:
+                  pass
+              finally:
+                  if change_stream is not None:
+                      change_stream.close()
+
+                  # Prevent "Task was destroyed but it is pending!"
+                  loop.run_until_complete(task)
+
+        The :class:`~MotorChangeStream` async iterable blocks
+        until the next change document is returned or an error is raised. If
+        the :meth:`~MotorChangeStream.next` method encounters
+        a network error when retrieving a batch from the server, it will
+        automatically attempt to recreate the cursor such that no change
+        events are missed. Any error encountered during the resume attempt
+        indicates there may be an outage and will be raised.
+
+        .. code-block:: python3
+
+            try:
+                pipeline = [{'$match': {'operationType': 'insert'}}]
+                async with db.collection.watch(pipeline) as stream:
+                    async for change in stream:
+                        print(change)
+            except pymongo.errors.PyMongoError:
+                # The ChangeStream encountered an unrecoverable error or the
+                # resume attempt failed to recreate the cursor.
+                logging.error('...')
+
+        For a precise description of the resume process see the
+        `change streams specification`_.
+
+        :Parameters:
+          - `pipeline` (optional): A list of aggregation pipeline stages to
+            append to an initial ``$changeStream`` stage. Not all
+            pipeline stages are valid after a ``$changeStream`` stage, see the
+            MongoDB documentation on change streams for the supported stages.
+          - `full_document` (optional): The fullDocument option to pass
+            to the ``$changeStream`` stage. Allowed values: 'default',
+            'updateLookup'.  Defaults to 'default'.
+            When set to 'updateLookup', the change notification for partial
+            updates will include both a delta describing the changes to the
+            document, as well as a copy of the entire document that was
+            changed from some time after the change occurred.
+          - `resume_after` (optional): The logical starting point for this
+            change stream.
+          - `max_await_time_ms` (optional): The maximum time in milliseconds
+            for the server to wait for changes before responding to a getMore
+            operation.
+          - `batch_size` (optional): The maximum number of documents to return
+            per batch.
+          - `collation` (optional): The :class:`~pymongo.collation.Collation`
+            to use for the aggregation.
+          - `session` (optional): a
+            :class:`~pymongo.client_session.ClientSession`.
+
+        :Returns:
+          A :class:`~MotorChangeStream`.
+
+        See the :ref:`tornado_change_stream_example`.
+
+        .. versionadded:: 1.2
+
+        .. mongodoc:: changeStreams
+
+        .. _change streams specification:
+            https://github.com/mongodb/specifications/blob/master/source/change-streams.rst
+        """
+        cursor_class = create_class_with_framework(
+            AgnosticChangeStream, self._framework, self.__module__)
+
+        # Latent cursor that will send initial command on first "async for".
+        return cursor_class(self, pipeline, full_document, resume_after,
+                            max_await_time_ms, batch_size, collation, session)
+
+    def list_indexes(self, session=None):
+        """Get a cursor over the index documents for this collection. ::
+
+          async def print_indexes():
+              for index in await db.test.list_indexes():
+                  print(index)
+
+        If the only index is the default index on ``_id``, this might print::
+
+            SON([('v', 1), ('key', SON([('_id', 1)])), ('name', '_id_'), ('ns', 'test.test')])
+        """
+        cursor_class = create_class_with_framework(
+            AgnosticLatentCommandCursor, self._framework, self.__module__)
+
+        # Latent cursor that will send initial command on first "async for".
+        return cursor_class(self, self._async_list_indexes, session=session)
 
     def parallel_scan(self, num_cursors, **kwargs):
         """Scan this entire collection in parallel.
@@ -461,6 +627,9 @@ class AgnosticCollection(AgnosticBaseProperties):
 
         :Parameters:
           - `num_cursors`: the number of cursors to return
+          - `session` (optional): a
+            :class:`~pymongo.client_session.ClientSession`, created with
+            :meth:`~MotorClient.start_session`.
 
         .. note:: Requires server version **>= 2.5.5**.
         """
@@ -565,6 +734,13 @@ class AgnosticCollection(AgnosticBaseProperties):
                 self.__module__)
 
             return command_cursor_class(obj, self)
+        elif obj.__class__ is ChangeStream:
+            change_stream_class = create_class_with_framework(
+                AgnosticChangeStream,
+                self._framework,
+                self.__module__)
+
+            return change_stream_class(obj, self)
         else:
             return obj
 
@@ -579,6 +755,7 @@ class AgnosticBaseCursor(AgnosticBase):
     cursor_id     = ReadOnlyProperty()
     alive         = ReadOnlyProperty()
     batch_size    = MotorCursorChainingMethod()
+    session       = ReadOnlyProperty()
 
     def __init__(self, cursor, collection):
         """Don't construct a cursor yourself, but acquire one from methods like
@@ -598,7 +775,7 @@ class AgnosticBaseCursor(AgnosticBase):
         self.closed = False
 
     # python.org/dev/peps/pep-0492/#api-design-and-implementation-revisions
-    if PY352:
+    if PY35:
         exec(textwrap.dedent("""
         def __aiter__(self):
             return self
@@ -610,19 +787,6 @@ class AgnosticBaseCursor(AgnosticBase):
             raise StopAsyncIteration()
         """), globals(), locals())
 
-    elif PY35:
-        # In Python 3.5.0 and 3.5.1, __aiter__ is a coroutine.
-        exec(textwrap.dedent("""
-        async def __aiter__(self):
-            return self
-
-        async def __anext__(self):
-            # An optimization: skip the "await" if possible.
-            if self._buffer_size() or await self.fetch_next:
-                return self.next_object()
-            raise StopAsyncIteration()
-        """), globals(), locals())
-
     def _get_more(self):
         """Initial query or getMore. Returns a Future."""
         if not self.alive:
@@ -928,7 +1092,7 @@ class AgnosticCursor(AgnosticBaseCursor):
     __delegate_class__ = Cursor
     address           = ReadOnlyProperty()
     count             = AsyncRead()
-    collation         = ReadOnlyProperty()
+    collation         = MotorCursorChainingMethod()
     distinct          = AsyncRead()
     explain           = AsyncRead()
     add_option        = MotorCursorChainingMethod()
@@ -1033,29 +1197,33 @@ class _LatentCursor(object):
         pass
 
 
-class AgnosticAggregationCursor(AgnosticCommandCursor):
-    __motor_class_name__ = 'MotorAggregationCursor'
+class AgnosticLatentCommandCursor(AgnosticCommandCursor):
+    __motor_class_name__ = 'MotorLatentCommandCursor'
 
-    def __init__(self, collection, pipeline, **kwargs):
+    def __init__(self, collection, start, *args, **kwargs):
         # We're being constructed without yield or await, like:
         #
         #     cursor = collection.aggregate(pipeline)
         #
         # ... so we can't send the "aggregate" command to the server and get
         # a PyMongo CommandCursor back yet. Set self.delegate to a latent
-        # cursor until the first yield or await triggers _get_more().
+        # cursor until the first yield or await triggers _get_more(), which
+        # will execute the callback "start", which gets a PyMongo CommandCursor.
         super(self.__class__, self).__init__(_LatentCursor(), collection)
-        self.pipeline = pipeline
+        self.start = start
+        self.args = args
         self.kwargs = kwargs
 
     def _get_more(self):
         if not self.started:
             self.started = True
             original_future = self._framework.get_future(self.get_io_loop())
-            future = self.collection._async_aggregate(
-                self.pipeline,
+            future = self.start(
+                *self.args,
                 **self.kwargs)
 
+            self.start = self.args = self.kwargs = None
+
             self._framework.add_future(
                 self.get_io_loop(),
                 future,
@@ -1093,3 +1261,99 @@ class AgnosticBulkOperationBuilder(AgnosticBase):
 
     def get_io_loop(self):
         return self.io_loop
+
+
+class AgnosticChangeStream(AgnosticBase):
+    """A change stream cursor.
+
+    Should not be called directly by application developers. See
+    :meth:`~MotorCollection.watch` for example usage.
+
+    .. versionadded: 1.2
+    .. mongodoc:: changeStreams
+    """
+    __delegate_class__ = ChangeStream
+    __motor_class_name__ = 'MotorChangeStream'
+
+    _close = AsyncCommand(attr_name='close')
+
+    def __init__(self, collection, pipeline, full_document, resume_after,
+                 max_await_time_ms, batch_size, collation, session):
+        super(self.__class__, self).__init__(delegate=None)
+        self._collection = collection
+        self._kwargs = {'pipeline': pipeline,
+                        'full_document': full_document,
+                        'resume_after': resume_after,
+                        'max_await_time_ms': max_await_time_ms,
+                        'batch_size': batch_size,
+                        'collation': collation,
+                        'session': session}
+
+    def _next(self):
+        # This method is run on a thread.
+        try:
+            if not self.delegate:
+                self.delegate = self._collection.delegate.watch(**self._kwargs)
+
+            return self.delegate.next()
+        except StopIteration:
+            raise StopAsyncIteration()
+
+    @coroutine_annotation(callback=False)
+    def next(self):
+        """Advance the cursor.
+
+        This method blocks until the next change document is returned or an
+        unrecoverable error is raised.
+
+        Raises :exc:`StopAsyncIteration` if this change stream is closed.
+
+        You can iterate the change stream by calling
+        ``await change_stream.next()`` repeatedly, or with an "async for" loop:
+
+        .. code-block:: python3
+
+          async for change in db.collection.watch():
+              print(change)
+
+        """
+        loop = self.get_io_loop()
+        return self._framework.run_on_executor(loop, self._next)
+
+    @coroutine_annotation(callback=False)
+    def close(self):
+        """Close this change stream.
+
+        Stops any "async for" loops using this change stream.
+        """
+        if self.delegate:
+            return self._close()
+
+        # Never started.
+        future = self._framework.get_future(self.get_io_loop())
+        future.set_result(None)
+        return future
+
+    if PY35:
+        exec(textwrap.dedent("""
+        async def __aiter__(self):
+            return self
+
+        __anext__ = next
+
+        async def __aenter__(self):
+            return self
+    
+        async def __aexit__(self, exc_type, exc_val, exc_tb):
+            if self.delegate:
+                self.delegate.close() 
+        """), globals(), locals())
+
+    def get_io_loop(self):
+        return self._collection.get_io_loop()
+
+    def __enter__(self):
+        raise RuntimeError('Use a change stream in "async with", not "with"')
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        pass
