@@ -16,14 +16,11 @@
 
 from __future__ import unicode_literals, absolute_import
 
-import warnings
-
 from . import core, motor_gridfs
 from .frameworks import tornado as tornado_framework
 from .metaprogramming import create_class_with_framework
 
-# See https://bugs.python.org/issue21720
-__all__ = list(map(str, ['MotorClient', 'Op']))
+__all__ = ['MotorClient']
 
 
 def create_motor_class(cls):
@@ -45,7 +42,10 @@ MotorCursor = create_motor_class(core.AgnosticCursor)
 MotorCommandCursor = create_motor_class(core.AgnosticCommandCursor)
 
 
-MotorAggregationCursor = create_motor_class(core.AgnosticAggregationCursor)
+MotorLatentCommandCursor = create_motor_class(core.AgnosticLatentCommandCursor)
+
+
+MotorChangeStream = create_motor_class(core.AgnosticChangeStream)
 
 
 MotorBulkOperationBuilder = create_motor_class(core.AgnosticBulkOperationBuilder)
@@ -64,17 +64,3 @@ MotorGridOut = create_motor_class(motor_gridfs.AgnosticGridOut)
 
 
 MotorGridOutCursor = create_motor_class(motor_gridfs.AgnosticGridOutCursor)
-
-
-def Op(fn, *args, **kwargs):
-    """Obsolete; here for backwards compatibility with Motor 0.1.
-
-    Op had been necessary for ease-of-use with Tornado 2 and @gen.engine. But
-    Motor 0.2 is built for Tornado 3, @gen.coroutine, and Futures, so motor.Op
-    is deprecated.
-    """
-    msg = "motor.Op is deprecated, simply call %s and yield its Future." % (
-        fn.__name__)
-
-    warnings.warn(msg, DeprecationWarning, stacklevel=2)
-    return fn(*args, **kwargs)
