@@ -35,6 +35,9 @@ from motor.metaprogramming import MotorAttributeFactory
 from gridfs.errors import *
 from pymongo import *
 from pymongo import (collation,
+                     change_stream,
+                     errors,
+                     monotonic,
                      operations,
                      server_selectors,
                      server_type,
@@ -43,10 +46,10 @@ from pymongo import (collation,
                      ssl_support,
                      write_concern)
 from pymongo.auth import _build_credentials_tuple
-from pymongo.helpers import _unpack_response, _check_command_response
+from pymongo.helpers import _check_command_response
 from pymongo.collation import *
 from pymongo.common import *
-from pymongo.common import _UUID_REPRESENTATIONS
+from pymongo.common import _UUID_REPRESENTATIONS, _MAX_END_SESSIONS
 from pymongo.cursor import *
 from pymongo.cursor import _QUERY_OPTIONS
 from pymongo.errors import *
@@ -56,12 +59,14 @@ from pymongo.message import (_COMMAND_OVERHEAD,
                              _maybe_add_read_preference)
 from pymongo.monitor import *
 from pymongo.monitoring import *
-from pymongo.monitoring import _LISTENERS, _Listeners
+from pymongo.monitoring import _LISTENERS, _Listeners, _SENSITIVE_COMMANDS
+from pymongo.monotonic import time
 from pymongo.operations import *
 from pymongo.pool import *
 from pymongo.pool import _METADATA
 from pymongo.periodic_executor import *
 from pymongo.periodic_executor import _EXECUTORS
+from pymongo.read_concern import *
 from pymongo.read_preferences import *
 from pymongo.read_preferences import _ServerMode
 from pymongo.results import *
@@ -74,7 +79,7 @@ from pymongo.son_manipulator import *
 from pymongo.topology import *
 from pymongo.topology_description import *
 from pymongo.uri_parser import *
-from pymongo.uri_parser import _partition, _rpartition
+from pymongo.uri_parser import _partition, _rpartition, _HAVE_DNSPYTHON
 from pymongo.write_concern import *
 from pymongo import auth
 from pymongo.auth import *
@@ -121,7 +126,7 @@ def wrap_synchro(fn):
         if isinstance(motor_obj, motor.MotorDatabase):
             client = MongoClient(delegate=motor_obj.client)
             return Database(client, motor_obj.name, delegate=motor_obj)
-        if isinstance(motor_obj, motor.motor_tornado.MotorAggregationCursor):
+        if isinstance(motor_obj, motor.motor_tornado.MotorLatentCommandCursor):
             return CommandCursor(motor_obj)
         if isinstance(motor_obj, motor.motor_tornado.MotorCommandCursor):
             return CommandCursor(motor_obj)
@@ -327,13 +332,15 @@ class Database(Synchro):
     __delegate_class__ = motor.MotorDatabase
     get_collection     = WrapOutgoing()
 
-    def __init__(self, client, name, delegate=None):
+    def __init__(self, client, name, **kwargs):
         assert isinstance(client, MongoClient), (
             "Expected MongoClient, got %s"
             % repr(client))
 
         self._client = client
-        self.delegate = delegate or client.delegate[name]
+        self.delegate = kwargs.get('delegate') or motor.MotorDatabase(
+            client.delegate, name, **kwargs)
+
         assert isinstance(self.delegate, motor.MotorDatabase), (
             "synchro.Database delegate must be MotorDatabase, not "
             " %s" % repr(self.delegate))
@@ -363,15 +370,17 @@ class Collection(Synchro):
     find                            = WrapOutgoing()
     initialize_unordered_bulk_op    = WrapOutgoing()
     initialize_ordered_bulk_op      = WrapOutgoing()
+    list_indexes                    = WrapOutgoing()
 
-    def __init__(self, database, name, delegate=None):
+    def __init__(self, database, name, **kwargs):
         if not isinstance(database, Database):
             raise TypeError(
                 "First argument to synchro Collection must be synchro "
                 "Database, not %s" % repr(database))
 
         self.database = database
-        self.delegate = delegate or database.delegate[name]
+        self.delegate = kwargs.get('delegate') or motor.MotorCollection(
+            self.database.delegate, name, **kwargs)
 
         if not isinstance(self.delegate, motor.MotorCollection):
             raise TypeError(
@@ -384,15 +393,21 @@ class Collection(Synchro):
         self.synchronize(cursor.delegate._get_more)()
         return cursor
 
+    def parallel_scan(self, *args, **kwargs):
+        # Motor does no I/O initially in parallel_scan() but PyMongo does.
+        return self.synchronize(self.delegate.parallel_scan)(*args, **kwargs)
+
     def __getattr__(self, name):
         # Access to collections with dotted names, like db.test.mike
         fullname = self.name + '.' + name
-        return Collection(self.database, fullname, getattr(self.delegate, name))
+        return Collection(self.database, fullname,
+                          delegate=getattr(self.delegate, name))
 
     def __getitem__(self, name):
-        # Access to collections with dotted names, like db.test.mike
+        # Access to collections with dotted names, like db.test['mike']
         fullname = self.name + '.' + name
-        return Collection(self.database, fullname, self.delegate[name])
+        return Collection(self.database, fullname,
+                          delegate=self.delegate[name])
 
 
 class Cursor(Synchro):
