@@ -31,8 +31,7 @@ from mockupdb import MockupDB
 
 from motor import motor_asyncio
 from test.assert_logs_backport import AssertLogsMixin
-from test.version import _parse_version_string, padded
-from test.test_environment import env, CLIENT_PEM
+from test.test_environment import env, CA_PEM, CLIENT_PEM
 
 
 class _TestMethodWrapper(object):
@@ -99,7 +98,8 @@ class AsyncIOTestCase(AssertLogsMixin, unittest.TestCase):
         self.loop.run_until_complete(self.collection.drop())
 
     def get_client_kwargs(self, **kwargs):
-        if env.mongod_validates_client_cert:
+        if env.mongod_started_with_ssl:
+            kwargs.setdefault('ssl_ca_certs', CA_PEM)
             kwargs.setdefault('ssl_certfile', CLIENT_PEM)
 
         kwargs.setdefault('ssl', env.mongod_started_with_ssl)
@@ -113,7 +113,7 @@ class AsyncIOTestCase(AssertLogsMixin, unittest.TestCase):
         Ignores self.ssl, you must pass 'ssl' argument.
         """
         return motor_asyncio.AsyncIOMotorClient(
-            uri or env.rs_uri,
+            uri or env.uri,
             *args,
             **self.get_client_kwargs(**kwargs))
 
@@ -152,6 +152,7 @@ class AsyncIOMockServerTestCase(AsyncIOTestCase):
     def client_server(self, *args, **kwargs):
         server = self.server(*args, **kwargs)
         client = motor_asyncio.AsyncIOMotorClient(server.uri, io_loop=self.loop)
+        self.addCleanup(client.close)
 
         return client, server
 
@@ -266,18 +267,6 @@ def server_is_mongos(client):
     return ismaster_response.get('msg') == 'isdbgrid'
 
 
-@asyncio.coroutine
-def version(client):
-    info = yield from client.server_info()
-    return _parse_version_string(info["version"])
-
-
-@asyncio.coroutine
-def at_least(client, min_version):
-    client_version = yield from version(client)
-    return client_version >= tuple(padded(min_version, 4))
-
-
 @asyncio.coroutine
 def skip_if_mongos(client):
     is_mongos = yield from server_is_mongos(client)
@@ -287,8 +276,4 @@ def skip_if_mongos(client):
 
 @asyncio.coroutine
 def remove_all_users(db):
-    version_check = yield from at_least(db.client, (2, 5, 4))
-    if version_check:
-        yield from db.command({"dropAllUsersFromDatabase": 1})
-    else:
-        yield from db.system.users.delete_many({})
+    yield from db.command({"dropAllUsersFromDatabase": 1})
