@@ -17,10 +17,8 @@
 import asyncio
 import os
 import unittest
-import warnings
 from unittest import SkipTest
 
-import bson
 import pymongo
 from bson import CodecOptions
 from pymongo import ReadPreference, WriteConcern
@@ -60,6 +58,9 @@ class TestAsyncIOClient(AsyncIOTestCase):
 
     @asyncio_test
     def test_unix_socket(self):
+        if env.mongod_started_with_ssl:
+            raise SkipTest("Server started with SSL")
+
         mongodb_socket = '/tmp/mongodb-%d.sock' % env.port
 
         if not os.access(mongodb_socket, os.R_OK):
@@ -143,7 +144,6 @@ class TestAsyncIOClient(AsyncIOTestCase):
 
     @asyncio_test(timeout=60)
     def test_high_concurrency(self):
-        return
         yield from self.make_test_data()
 
         concurrency = 25
@@ -209,20 +209,15 @@ class TestAsyncIOClient(AsyncIOTestCase):
                 'mike', 'password',
                 roles=['userAdmin', 'readWrite'])
 
-            client = motor_asyncio.AsyncIOMotorClient(
-                'mongodb://u:pass@%s:%d' % (env.host, env.port),
-                io_loop=self.loop)
-
-            # ismaster doesn't throw auth errors.
-            yield from client.admin.command('ismaster')
+            client = self.asyncio_client(
+                'mongodb://u:pass@%s:%d' % (env.host, env.port))
 
             with self.assertRaises(OperationFailure):
                 yield from client.db.collection.find_one()
 
-            client = motor_asyncio.AsyncIOMotorClient(
+            client = self.asyncio_client(
                 'mongodb://mike:password@%s:%d/%s' %
-                (env.host, env.port, db.name),
-                io_loop=self.loop)
+                (env.host, env.port, db.name))
 
             yield from client[db.name].collection.find_one()
         finally:
@@ -233,12 +228,12 @@ class TestAsyncIOClient(AsyncIOTestCase):
         # Connect.
         yield from self.cx.server_info()
         ka = get_primary_pool(self.cx).opts.socket_keepalive
-        self.assertFalse(ka)
+        self.assertTrue(ka)
 
-        client = self.asyncio_client(socketKeepAlive=True)
+        client = self.asyncio_client(socketKeepAlive=False)
         yield from client.server_info()
         ka = get_primary_pool(client).opts.socket_keepalive
-        self.assertTrue(ka)
+        self.assertFalse(ka)
 
     def test_get_database(self):
         codec_options = CodecOptions(tz_aware=True)
@@ -252,6 +247,26 @@ class TestAsyncIOClient(AsyncIOTestCase):
         self.assertEqual(ReadPreference.SECONDARY, db.read_preference)
         self.assertEqual(write_concern, db.write_concern)
 
+    @asyncio_test
+    def test_list_databases(self):
+        yield from self.collection.insert_one({})
+        cursor = yield from self.cx.list_databases()
+        self.assertIsInstance(cursor, motor_asyncio.AsyncIOMotorCommandCursor)
+
+        while (yield from cursor.fetch_next):
+            info = cursor.next_object()
+            if info['name'] == self.collection.database.name:
+                break
+        else:
+            self.fail("'%s' database not found" % self.collection.database.name)
+
+    @asyncio_test
+    def test_list_database_names(self):
+        yield from self.collection.insert_one({})
+        names = yield from self.cx.list_database_names()
+        self.assertIsInstance(names, list)
+        self.assertIn(self.collection.database.name, names)
+
 
 class TestAsyncIOClientTimeout(AsyncIOMockServerTestCase):
     @asyncio_test
