@@ -18,7 +18,6 @@ import asyncio
 import sys
 import traceback
 import unittest
-from unittest import SkipTest
 
 import bson
 from bson import CodecOptions
@@ -34,9 +33,8 @@ from motor.motor_asyncio import (AsyncIOMotorCollection,
 import test
 from test.asyncio_tests import (asyncio_test,
                                 AsyncIOTestCase,
-                                skip_if_mongos,
-                                at_least)
-from test.utils import delay, ignore_deprecations
+                                skip_if_mongos)
+from test.utils import ignore_deprecations
 
 
 class TestAsyncIOCollection(AsyncIOTestCase):
@@ -88,41 +86,6 @@ class TestAsyncIOCollection(AsyncIOTestCase):
         else:
             self.fail('Expected TypeError')
 
-    @asyncio_test(timeout=30)
-    def test_find_is_async(self):
-        # Need parallel Javascript.
-        if not (yield from at_least(self.cx, (3,))):
-            raise SkipTest("Requires MongoDB >= 3.0")
-
-        # Confirm find() is async by launching two operations which will finish
-        # out of order. Also test that AsyncIOMotorClient doesn't reuse sockets
-        # incorrectly.
-
-        # Launch find operations for _id's 1 and 2 which will finish in order
-        # 2, then 1.
-        coll = self.collection
-        yield from coll.insert_many([{'_id': 1}, {'_id': 2}])
-        results = []
-
-        futures = [asyncio.Future(loop=self.loop),
-                   asyncio.Future(loop=self.loop)]
-
-        def callback(result, error):
-            if result:
-                results.append(result)
-                futures.pop().set_result(None)
-
-        # This find() takes 0.5 seconds.
-        coll.find({'_id': 1, '$where': delay(0.5)}).limit(1).each(callback)
-
-        # Very fast lookup.
-        coll.find({'_id': 2}).limit(1).each(callback)
-
-        yield from asyncio.gather(*futures, loop=self.loop)
-
-        # Results were appended in order 2, 1.
-        self.assertEqual([{'_id': 2}, {'_id': 1}], results)
-
     @ignore_deprecations
     @asyncio_test
     def test_update(self):
@@ -273,7 +236,7 @@ class TestAsyncIOCollection(AsyncIOTestCase):
 
         coll.database.client.close()
 
-    @asyncio_test
+    @asyncio_test(timeout=30)
     def test_nested_callbacks(self):
         results = [0]
         future = asyncio.Future(loop=self.loop)
@@ -377,11 +340,8 @@ class TestAsyncIOCollection(AsyncIOTestCase):
 
     pipeline = [{'$project': {'_id': '$_id'}}]
 
-    @asyncio_test
+    @asyncio_test(timeout=30)
     def test_aggregation_cursor(self):
-        if not (yield from at_least(self.cx, (2, 6))):
-            raise SkipTest("Requires MongoDB >= 2.6")
-
         db = self.db
 
         # A small collection which returns only an initial batch,
@@ -395,9 +355,6 @@ class TestAsyncIOCollection(AsyncIOTestCase):
 
     @asyncio_test
     def test_aggregation_cursor_exc_info(self):
-        if not (yield from at_least(self.cx, (2, 6))):
-            raise SkipTest("Requires MongoDB >= 2.6")
-
         yield from self._make_test_data(200)
         cursor = self.db.test.aggregate(self.pipeline)
         yield from cursor.to_list(length=10)
@@ -414,9 +371,6 @@ class TestAsyncIOCollection(AsyncIOTestCase):
 
     @asyncio_test(timeout=30)
     def test_parallel_scan(self):
-        if not (yield from at_least(self.cx, (2, 5, 5))):
-            raise SkipTest("Requires MongoDB >= 2.5.5")
-
         yield from skip_if_mongos(self.cx)
 
         collection = self.collection.with_options(
