@@ -15,7 +15,6 @@
 """Test AsyncIOMotorCursor."""
 
 import asyncio
-import gc
 import sys
 import traceback
 import unittest
@@ -25,7 +24,6 @@ from unittest import SkipTest
 from pymongo import CursorType
 from pymongo.errors import InvalidOperation, ExecutionTimeout
 from pymongo.errors import OperationFailure
-from mockupdb import OpQuery, OpKillCursors
 
 from motor import motor_asyncio
 from test.utils import one, safe_get, get_primary_pool
@@ -33,7 +31,6 @@ from test.asyncio_tests import (asyncio_test,
                                 AsyncIOTestCase,
                                 AsyncIOMockServerTestCase,
                                 server_is_mongos,
-                                at_least,
                                 get_command_line)
 
 
@@ -81,20 +78,26 @@ class TestAsyncIOCursor(AsyncIOMockServerTestCase):
         self.assertEqual(200, i)
 
     @unittest.skipUnless(sys.version_info >= (3, 4), "Python 3.4 required")
+    @unittest.skipIf('PyPy' in sys.version, "PyPy")
     @asyncio_test
     def test_fetch_next_delete(self):
-        client, server = self.client_server(auto_ismaster={'ismaster': True})
+        client, server = self.client_server(auto_ismaster=True)
 
-        cursor = client.test.collection.find()
+        cursor = client.test.coll.find()
         self.fetch_next(cursor)
-        request = yield from self.run_thread(server.receives, OpQuery)
-        request.replies({'_id': 1}, cursor_id=123)
+        request = yield from self.run_thread(server.receives, "find", "coll")
+        request.replies({"cursor": {
+            "id": 123,
+            "ns": "db.coll",
+            "firstBatch": [{"_id": 1}]}})
 
         # Decref the cursor and clear from the event loop.
         del cursor
         yield
-        yield from self.run_thread(server.receives,
-                                   OpKillCursors(cursor_ids=[123]))
+        request = yield from self.run_thread(
+            server.receives, "killCursors", "coll")
+
+        request.ok()
 
     @asyncio_test
     def test_fetch_next_without_results(self):
@@ -233,22 +236,28 @@ class TestAsyncIOCursor(AsyncIOMockServerTestCase):
 
     @asyncio_test
     def test_cursor_explicit_close(self):
-        client, server = self.client_server(auto_ismaster={'ismaster': True})
-        collection = client.test.collection
+        client, server = self.client_server(auto_ismaster=True)
+        collection = client.test.coll
         cursor = collection.find()
 
         future = self.fetch_next(cursor)
         self.assertTrue(cursor.alive)
-        request = yield from self.run_thread(server.receives, OpQuery)
-        request.replies({'_id': 1}, cursor_id=123)
+        request = yield from self.run_thread(server.receives, "find", "coll")
+        request.replies({"cursor": {
+            "id": 123,
+            "ns": "db.coll",
+            "firstBatch": [{"_id": 1}]}})
+
         self.assertTrue((yield from future))
         self.assertEqual(123, cursor.cursor_id)
 
         future = self.ensure_future(cursor.close())
 
         # No reply to OP_KILLCURSORS.
-        yield from self.run_thread(server.receives,
-                                   OpKillCursors(cursor_ids=[123]))
+        request = yield from self.run_thread(
+            server.receives, "killCursors", "coll")
+
+        request.ok()
         yield from future
 
         # Cursor reports it's alive because it has buffered data, even though
@@ -333,14 +342,18 @@ class TestAsyncIOCursor(AsyncIOMockServerTestCase):
         self.assertEqual(cursor, cursor.rewind())
 
     @unittest.skipUnless(sys.version_info >= (3, 4), "Python 3.4 required")
+    @unittest.skipIf("PyPy" in sys.version, "PyPy")
     @asyncio_test
     def test_cursor_del(self):
         client, server = self.client_server(auto_ismaster=True)
-        cursor = client.test.collection.find()
+        cursor = client.test.coll.find()
 
         future = self.fetch_next(cursor)
-        request = yield from self.run_thread(server.receives, OpQuery)
-        request.replies({'_id': 1}, cursor_id=123)
+        request = yield from self.run_thread(server.receives, "find", "coll")
+        request.replies({"cursor": {
+            "id": 123,
+            "ns": "db.coll",
+            "firstBatch": [{"_id": 1}]}})
         yield from future  # Complete the first fetch.
 
         # Dereference the cursor.
@@ -349,10 +362,10 @@ class TestAsyncIOCursor(AsyncIOMockServerTestCase):
         # Let the event loop iterate once more to clear its references to
         # callbacks, allowing the cursor to be freed.
         yield from asyncio.sleep(0, loop=self.loop)
-        if 'PyPy' in sys.version:
-            gc.collect()
+        request = yield from self.run_thread(
+            server.receives, "killCursors", "coll")
 
-        yield from self.run_thread(server.receives, OpKillCursors)
+        request.ok()
 
     @unittest.skipUnless(sys.version_info >= (3, 4), "Python 3.4 required")
     @asyncio_test
@@ -414,7 +427,7 @@ class TestAsyncIOCursor(AsyncIOMockServerTestCase):
         self.assertEqual(0, len(socks))
         if 'PyPy' in sys.version:
             # Don't wait for GC or use gc.collect(), it's unreliable.
-            cur.close()
+            yield from cur.close()
 
         del cur
 
@@ -454,9 +467,6 @@ class TestAsyncIOCursorMaxTimeMS(AsyncIOTestCase):
         if (yield from server_is_mongos(self.cx)):
             raise SkipTest("mongos has no maxTimeAlwaysTimeOut fail point")
 
-        if not (yield from at_least(self.cx, (2, 5, 3, -1))):
-            raise SkipTest("maxTimeMS requires MongoDB >= 2.5.3")
-
         cmdline = yield from get_command_line(self.cx)
         if '1' != safe_get(cmdline, 'parsed.setParameter.enableTestCommands'):
             if 'enableTestCommands=1' not in cmdline['argv']:
