@@ -16,28 +16,26 @@ from __future__ import unicode_literals
 
 """Test Motor, an asynchronous driver for MongoDB and Tornado."""
 
-import gc
 import sys
 import traceback
 import unittest
 import warnings
 
 import pymongo
-from mockupdb import OpQuery
-from mockupdb import OpKillCursors
 from tornado import gen
 from tornado.concurrent import Future
 from tornado.testing import gen_test
-from tornado import version_info as tornado_version
 from pymongo import CursorType
+from pymongo.collation import Collation
 from pymongo.errors import InvalidOperation, ExecutionTimeout
 from pymongo.errors import OperationFailure
 
 import motor
 import motor.motor_tornado
-from test import SkipTest
-from test.tornado_tests import (at_least, get_command_line,
-                                MotorTest, MotorMockServerTest,
+from test import SkipTest, env
+from test.tornado_tests import (get_command_line,
+                                MotorTest,
+                                MotorMockServerTest,
                                 server_is_mongos)
 from test.utils import one, safe_get, get_primary_pool
 
@@ -48,10 +46,6 @@ class MotorCursorTest(MotorMockServerTest):
         self.assertTrue(isinstance(cursor, motor.motor_tornado.MotorCursor))
         self.assertFalse(cursor.started, "Cursor shouldn't start immediately")
 
-    @gen_test
-    def test_count_callback(self):
-        yield self.check_optional_callback(self.collection.find().count)
-
     @gen_test
     def test_count(self):
         yield self.make_test_data()
@@ -103,20 +97,23 @@ class MotorCursorTest(MotorMockServerTest):
         if 'PyPy' in sys.version:
             raise SkipTest('PyPy')
 
-        client, server = self.client_server(auto_ismaster={'ismaster': True})
-        cursor = client.test.collection.find()
+        client, server = self.client_server(auto_ismaster=True)
+        cursor = client.test.coll.find()
 
         # With Tornado, simply accessing fetch_next starts the fetch.
         cursor.fetch_next
-        request = yield self.run_thread(server.receives, OpQuery)
-        request.replies({'_id': 1}, cursor_id=123)
+        request = yield self.run_thread(server.receives, "find", "coll")
+        request.replies({"cursor": {
+            "id": 123,
+            "ns": "db.coll",
+            "firstBatch": [{"_id": 1}]}})
 
         # Decref'ing the cursor eventually closes it on the server.
         del cursor
-        # Clear Runner's reference. This is Tornado 3 substitute for gen.moment.
-        yield gen.Task(self.io_loop.add_callback)
-        yield self.run_thread(server.receives,
-                              OpKillCursors(cursor_ids=[123]))
+        # Clear Runner's reference.
+        yield gen.moment
+        request = yield self.run_thread(server.receives, "killCursors", "coll")
+        request.ok()
 
     @gen_test
     def test_fetch_next_without_results(self):
@@ -196,8 +193,6 @@ class MotorCursorTest(MotorMockServerTest):
         yield self.make_test_data()
         coll = self.collection
         cursor = coll.find()
-        yield self.check_optional_callback(cursor.to_list, 10)
-        cursor = coll.find()
         with self.assertRaises(ValueError):
             yield cursor.to_list(-1)
 
@@ -284,20 +279,36 @@ class MotorCursorTest(MotorMockServerTest):
         with self.assertRaises(InvalidOperation):
             yield cursor.to_list(10)
 
+    @env.require_version_min(3, 4)
+    @gen_test
+    def test_to_list_with_chained_collation(self):
+        yield self.make_test_data()
+        cursor = self.collection.find({}, {'_id': 1}) \
+            .sort([('_id', pymongo.ASCENDING)]) \
+            .collation(Collation("en"))
+        expected = [{'_id': i} for i in range(200)]
+        result = yield cursor.to_list(length=1000)
+        self.assertEqual(expected, result)
+
     @gen_test
     def test_cursor_explicit_close(self):
-        client, server = self.client_server(auto_ismaster={'ismaster': True})
-        collection = client.test.collection
+        client, server = self.client_server(auto_ismaster=True)
+        collection = client.test.coll
         cursor = collection.find()
 
         # With Tornado, simply accessing fetch_next starts the fetch.
         fetch_next = cursor.fetch_next
-        request = yield self.run_thread(server.receives, OpQuery)
-        request.replies({'_id': 1}, cursor_id=123)
+        request = yield self.run_thread(server.receives, "find", "coll")
+        request.replies({"cursor": {
+            "id": 123,
+            "ns": "db.coll",
+            "firstBatch": [{"_id": 1}]}})
+
         self.assertTrue((yield fetch_next))
 
         close_future = cursor.close()
-        yield self.run_thread(server.receives, OpKillCursors(cursor_ids=[123]))
+        request = yield self.run_thread(server.receives, "killCursors", "coll")
+        request.ok()
         yield close_future
 
         # Cursor reports it's alive because it has buffered data, even though
@@ -386,15 +397,18 @@ class MotorCursorTest(MotorMockServerTest):
         if sys.version_info < (3, 4):
             raise SkipTest("requires Python 3.4")
 
-        if tornado_version < (4, 0, 0, 0):
-            raise SkipTest("Tornado 3")
+        if 'PyPy' in sys.version:
+            raise SkipTest("PyPy")
 
         client, server = self.client_server(auto_ismaster=True)
-        cursor = client.test.collection.find()
+        cursor = client.test.coll.find()
 
         future = cursor.fetch_next
-        request = yield self.run_thread(server.receives, OpQuery)
-        request.replies({'_id': 1}, cursor_id=123)
+        request = yield self.run_thread(server.receives, "find", "coll")
+        request.replies({"cursor": {
+            "id": 123,
+            "ns": "db.coll",
+            "firstBatch": [{"_id": 1}]}})
         yield future  # Complete the first fetch.
 
         # Dereference the cursor.
@@ -402,11 +416,9 @@ class MotorCursorTest(MotorMockServerTest):
 
         # Let the event loop iterate once more to clear its references to
         # callbacks, allowing the cursor to be freed.
-        yield self.pause(0)
-        if 'PyPy' in sys.version:
-            gc.collect()
-
-        yield self.run_thread(server.receives, OpKillCursors)
+        yield gen.sleep(0.1)
+        request = yield self.run_thread(server.receives, "killCursors", "coll")
+        request.ok()
 
     @gen_test
     def test_exhaust(self):
@@ -466,11 +478,11 @@ class MotorCursorTest(MotorMockServerTest):
         self.assertEqual(0, len(socks))
         if 'PyPy' in sys.version:
             # Don't wait for GC or use gc.collect(), it's unreliable.
-            cur.close()
+            yield cur.close()
 
         del cur
 
-        yield self.pause(0.1)
+        yield gen.sleep(0.1)
 
         # The exhaust cursor's socket was discarded, although another may
         # already have been opened to send OP_KILLCURSORS.
@@ -512,9 +524,6 @@ class MotorCursorMaxTimeMSTest(MotorTest):
         if (yield server_is_mongos(self.cx)):
             raise SkipTest("mongos has no maxTimeAlwaysTimeOut fail point")
 
-        if not (yield at_least(self.cx, (2, 5, 3, -1))):
-            raise SkipTest("maxTimeMS requires MongoDB >= 2.5.3")
-
         cmdline = yield get_command_line(self.cx)
         if '1' != safe_get(cmdline, 'parsed.setParameter.enableTestCommands'):
             if 'enableTestCommands=1' not in cmdline['argv']:
